@@ -7,19 +7,19 @@
 A Dual Annealing global optimization algorithm
 """
 
-from __future__ import division, print_function, absolute_import
+import warnings
 
 import numpy as np
 from scipy.optimize import OptimizeResult
-from scipy.optimize import minimize
+from scipy.optimize import minimize, Bounds
 from scipy.special import gammaln
 from scipy._lib._util import check_random_state
-
+from scipy.optimize._constraints import new_bounds_to_old
 
 __all__ = ['dual_annealing']
 
 
-class VisitingDistribution(object):
+class VisitingDistribution:
     """
     Class used to generate new coordinates based on the distorted
     Cauchy-Lorentz distribution. Depending on the steps within the strategy
@@ -29,31 +29,32 @@ class VisitingDistribution(object):
     Parameters
     ----------
     lb : array_like
-        A 1-D numpy ndarray containing lower bounds of the generated
+        A 1-D NumPy ndarray containing lower bounds of the generated
         components. Neither NaN or inf are allowed.
     ub : array_like
-        A 1-D numpy ndarray containing upper bounds for the generated
+        A 1-D NumPy ndarray containing upper bounds for the generated
         components. Neither NaN or inf are allowed.
     visiting_param : float
         Parameter for visiting distribution. Default value is 2.62.
         Higher values give the visiting distribution a heavier tail, this
         makes the algorithm jump to a more distant region.
-        The value range is (0, 3]. It's value is fixed for the life of the
+        The value range is (1, 3]. Its value is fixed for the life of the
         object.
-    rand_state : `~numpy.random.mtrand.RandomState` object
-        A `~numpy.random.mtrand.RandomState` object for using the current state
-        of the created random generator container.
+    rand_gen : {`~numpy.random.RandomState`, `~numpy.random.Generator`}
+        A `~numpy.random.RandomState`, `~numpy.random.Generator` object
+        for using the current state of the created random generator container.
+
     """
     TAIL_LIMIT = 1.e8
     MIN_VISIT_BOUND = 1.e-10
 
-    def __init__(self, lb, ub, visiting_param, rand_state):
+    def __init__(self, lb, ub, visiting_param, rand_gen):
         # if you wish to make _visiting_param adjustable during the life of
         # the object then _factor2, _factor3, _factor5, _d1, _factor6 will
         # have to be dynamically calculated in `visit_fn`. They're factored
         # out here so they don't need to be recalculated all the time.
         self._visiting_param = visiting_param
-        self.rand_state = rand_state
+        self.rand_gen = rand_gen
         self.lower = lb
         self.upper = ub
         self.bound_range = ub - lb
@@ -72,7 +73,7 @@ class VisitingDistribution(object):
             np.pi * (1.0 - self._factor5)) / np.exp(gammaln(self._d1))
 
     def visiting(self, x, step, temperature):
-        """ Based on the step in the strategy chain, new coordinated are
+        """ Based on the step in the strategy chain, new coordinates are
         generated by changing all components is the same time or only
         one of them, the new values are computed with visit_fn method
         """
@@ -80,8 +81,7 @@ class VisitingDistribution(object):
         if step < dim:
             # Changing all coordinates with a new visiting value
             visits = self.visit_fn(temperature, dim)
-            upper_sample = self.rand_state.random_sample()
-            lower_sample = self.rand_state.random_sample()
+            upper_sample, lower_sample = self.rand_gen.uniform(size=2)
             visits[visits > self.TAIL_LIMIT] = self.TAIL_LIMIT * upper_sample
             visits[visits < -self.TAIL_LIMIT] = -self.TAIL_LIMIT * lower_sample
             x_visit = visits + x
@@ -94,11 +94,11 @@ class VisitingDistribution(object):
             # Changing only one coordinate at a time based on strategy
             # chain step
             x_visit = np.copy(x)
-            visit = self.visit_fn(temperature, 1)
+            visit = self.visit_fn(temperature, 1)[0]
             if visit > self.TAIL_LIMIT:
-                visit = self.TAIL_LIMIT * self.rand_state.random_sample()
+                visit = self.TAIL_LIMIT * self.rand_gen.uniform()
             elif visit < -self.TAIL_LIMIT:
-                visit = -self.TAIL_LIMIT * self.rand_state.random_sample()
+                visit = -self.TAIL_LIMIT * self.rand_gen.uniform()
             index = step - dim
             x_visit[index] = visit + x[index]
             a = x_visit[index] - self.lower[index]
@@ -112,7 +112,7 @@ class VisitingDistribution(object):
 
     def visit_fn(self, temperature, dim):
         """ Formula Visita from p. 405 of reference [2] """
-        x, y = self.rand_state.normal(size=(dim, 2)).T
+        x, y = self.rand_gen.normal(size=(dim, 2)).T
 
         factor1 = np.exp(np.log(temperature) / (self._visiting_param - 1.0))
         factor4 = self._factor4_p * factor1
@@ -127,7 +127,7 @@ class VisitingDistribution(object):
         return x / den
 
 
-class EnergyState(object):
+class EnergyState:
     """
     Class used to record the energy state. At any time, it knows what is the
     currently used coordinates and the most recent best location.
@@ -135,10 +135,10 @@ class EnergyState(object):
     Parameters
     ----------
     lower : array_like
-        A 1-D numpy ndarray containing lower bounds for generating an initial
+        A 1-D NumPy ndarray containing lower bounds for generating an initial
         random components in the `reset` method.
     upper : array_like
-        A 1-D numpy ndarray containing upper bounds for generating an initial
+        A 1-D NumPy ndarray containing upper bounds for generating an initial
         random components in the `reset` method
         components. Neither NaN or inf are allowed.
     callback : callable, ``callback(x, f, context)``, optional
@@ -146,7 +146,7 @@ class EnergyState(object):
         ``x`` and ``f`` are the coordinates and function value of the
         latest minimum found, and `context` has value in [0, 1, 2]
     """
-    # Maximimum number of trials for generating a valid starting point
+    # Maximum number of trials for generating a valid starting point
     MAX_REINIT_COUNT = 1000
 
     def __init__(self, lower, upper, callback=None):
@@ -158,14 +158,14 @@ class EnergyState(object):
         self.upper = upper
         self.callback = callback
 
-    def reset(self, func_wrapper, rand_state, x0=None):
+    def reset(self, func_wrapper, rand_gen, x0=None):
         """
         Initialize current location is the search domain. If `x0` is not
         provided, a random location within the bounds is generated.
         """
         if x0 is None:
-            self.current_location = self.lower + rand_state.random_sample(
-                len(self.lower)) * (self.upper - self.lower)
+            self.current_location = rand_gen.uniform(self.lower, self.upper,
+                                                     size=len(self.lower))
         else:
             self.current_location = np.copy(x0)
         init_error = True
@@ -184,8 +184,9 @@ class EnergyState(object):
                         'trying new random parameters'
                     )
                     raise ValueError(message)
-                self.current_location = self.lower + rand_state.random_sample(
-                    self.lower.size) * (self.upper - self.lower)
+                self.current_location = rand_gen.uniform(self.lower,
+                                                         self.upper,
+                                                         size=self.lower.size)
                 reinit_counter += 1
             else:
                 init_error = False
@@ -210,7 +211,7 @@ class EnergyState(object):
         self.current_location = np.copy(x)
 
 
-class StrategyChain(object):
+class StrategyChain:
     """
     Class that implements within a Markov chain the strategy for location
     acceptance and local search decision making.
@@ -228,14 +229,21 @@ class StrategyChain(object):
         Instance of `ObjectiveFunWrapper` class.
     minimizer_wrapper: LocalSearchWrapper
         Instance of `LocalSearchWrapper` class.
-    rand_state : `~numpy.random.mtrand.RandomState` object
-        A `~numpy.random.mtrand.RandomState` object for using the current state
-        of the created random generator container.
+    rand_gen : {None, int, `numpy.random.Generator`,
+                `numpy.random.RandomState`}, optional
+
+        If `seed` is None (or `np.random`), the `numpy.random.RandomState`
+        singleton is used.
+        If `seed` is an int, a new ``RandomState`` instance is used,
+        seeded with `seed`.
+        If `seed` is already a ``Generator`` or ``RandomState`` instance then
+        that instance is used.
     energy_state: EnergyState
         Instance of `EnergyState` class.
+
     """
     def __init__(self, acceptance_param, visit_dist, func_wrapper,
-                 minimizer_wrapper, rand_state, energy_state):
+                 minimizer_wrapper, rand_gen, energy_state):
         # Local strategy chain minimum energy and location
         self.emin = energy_state.current_energy
         self.xmin = np.array(energy_state.current_location)
@@ -251,20 +259,20 @@ class StrategyChain(object):
         self.minimizer_wrapper = minimizer_wrapper
         self.not_improved_idx = 0
         self.not_improved_max_idx = 1000
-        self._rand_state = rand_state
+        self._rand_gen = rand_gen
         self.temperature_step = 0
         self.K = 100 * len(energy_state.current_location)
 
     def accept_reject(self, j, e, x_visit):
-        r = self._rand_state.random_sample()
-        pqv_temp = (self.acceptance_param - 1.0) * (
-            e - self.energy_state.current_energy) / (
-                self.temperature_step + 1.)
+        r = self._rand_gen.uniform()
+        pqv_temp = 1.0 - ((1.0 - self.acceptance_param) *
+            (e - self.energy_state.current_energy) / self.temperature_step)
         if pqv_temp <= 0.:
             pqv = 0.
         else:
             pqv = np.exp(np.log(pqv_temp) / (
                 1. - self.acceptance_param))
+
         if r <= pqv:
             # We accept the new location and update state
             self.energy_state.update_current(e, x_visit)
@@ -332,7 +340,7 @@ class StrategyChain(object):
             pls = np.exp(self.K * (
                 self.energy_state.ebest - self.energy_state.current_energy) /
                 self.temperature_step)
-            if pls >= self._rand_state.random_sample():
+            if pls >= self._rand_gen.uniform():
                 do_ls = True
         # Global energy not improved, let's see what LS gives
         # on the best strategy chain location
@@ -356,7 +364,7 @@ class StrategyChain(object):
                         'during dual annealing')
 
 
-class ObjectiveFunWrapper(object):
+class ObjectiveFunWrapper:
 
     def __init__(self, func, maxfun=1e7, *args):
         self.func = func
@@ -374,7 +382,7 @@ class ObjectiveFunWrapper(object):
         return self.func(x, *self.args)
 
 
-class LocalSearchWrapper(object):
+class LocalSearchWrapper:
     """
     Class used to wrap around the minimizer used for local search
     Default local minimizer is SciPy minimizer L-BFGS-B
@@ -384,11 +392,11 @@ class LocalSearchWrapper(object):
     LS_MAXITER_MIN = 100
     LS_MAXITER_MAX = 1000
 
-    def __init__(self, bounds, func_wrapper, **kwargs):
+    def __init__(self, search_bounds, func_wrapper, **kwargs):
         self.func_wrapper = func_wrapper
         self.kwargs = kwargs
         self.minimizer = minimize
-        bounds_list = list(zip(*bounds))
+        bounds_list = list(zip(*search_bounds))
         self.lower = np.array(bounds_list[0])
         self.upper = np.array(bounds_list[1])
 
@@ -408,9 +416,9 @@ class LocalSearchWrapper(object):
         # Run local search from the given x location where energy value is e
         x_tmp = np.copy(x)
         mres = self.minimizer(self.func_wrapper.fun, x, **self.kwargs)
-        if 'njev' in mres.keys():
+        if 'njev' in mres:
             self.func_wrapper.ngev += mres.njev
-        if 'nhev' in mres.keys():
+        if 'nhev' in mres:
             self.func_wrapper.nhev += mres.nhev
         # Check if is valid value
         is_finite = np.all(np.isfinite(mres.x)) and np.isfinite(mres.fun)
@@ -426,29 +434,32 @@ class LocalSearchWrapper(object):
 
 
 def dual_annealing(func, bounds, args=(), maxiter=1000,
-                   local_search_options={}, initial_temp=5230.,
+                   minimizer_kwargs=None, initial_temp=5230.,
                    restart_temp_ratio=2.e-5, visit=2.62, accept=-5.0,
                    maxfun=1e7, seed=None, no_local_search=False,
-                   callback=None, x0=None):
+                   callback=None, x0=None, local_search_options=None):
     """
     Find the global minimum of a function using Dual Annealing.
 
     Parameters
     ----------
     func : callable
-        The objective function to be minimized.  Must be in the form
+        The objective function to be minimized. Must be in the form
         ``f(x, *args)``, where ``x`` is the argument in the form of a 1-D array
         and ``args`` is a  tuple of any additional fixed parameters needed to
         completely specify the function.
-    bounds : sequence, shape (n, 2)
-        Bounds for variables.  ``(min, max)`` pairs for each element in ``x``,
-        defining bounds for the objective function parameter.
+    bounds : sequence or `Bounds`
+        Bounds for variables. There are two ways to specify the bounds:
+
+        1. Instance of `Bounds` class.
+        2. Sequence of ``(min, max)`` pairs for each element in `x`.
+
     args : tuple, optional
         Any additional fixed parameters needed to completely specify the
         objective function.
     maxiter : int, optional
         The maximum number of global search iterations. Default value is 1000.
-    local_search_options : dict, optional
+    minimizer_kwargs : dict, optional
         Extra keyword arguments to be passed to the local minimizer
         (`minimize`). Some important options could be:
         ``method`` for the minimizer method to use and ``args`` for
@@ -465,7 +476,7 @@ def dual_annealing(func, bounds, args=(), maxiter=1000,
     visit : float, optional
         Parameter for visiting distribution. Default value is 2.62. Higher
         values give the visiting distribution a heavier tail, this makes
-        the algorithm jump to a more distant region. The value range is (0, 3].
+        the algorithm jump to a more distant region. The value range is (1, 3].
     accept : float, optional
         Parameter for acceptance distribution. It is used to control the
         probability of acceptance. The lower the acceptance parameter, the
@@ -476,16 +487,18 @@ def dual_annealing(func, bounds, args=(), maxiter=1000,
         algorithm is in the middle of a local search, this number will be
         exceeded, the algorithm will stop just after the local search is
         done. Default value is 1e7.
-    seed : {int or `~numpy.random.mtrand.RandomState` instance}, optional
-        If `seed` is not specified the `~numpy.random.mtrand.RandomState`
+    seed : {None, int, `numpy.random.Generator`,
+            `numpy.random.RandomState`}, optional
+
+        If `seed` is None (or `np.random`), the `numpy.random.RandomState`
         singleton is used.
         If `seed` is an int, a new ``RandomState`` instance is used,
         seeded with `seed`.
-        If `seed` is already a ``RandomState`` instance, then that
-        instance is used.
+        If `seed` is already a ``Generator`` or ``RandomState`` instance then
+        that instance is used.
         Specify `seed` for repeatable minimizations. The random numbers
-        generated with this seed only affect the visiting distribution
-        function and new coordinates generation.
+        generated with this seed only affect the visiting distribution function
+        and new coordinates generation.
     no_local_search : bool, optional
         If `no_local_search` is set to True, a traditional Generalized
         Simulated Annealing will be performed with no local search
@@ -498,12 +511,19 @@ def dual_annealing(func, bounds, args=(), maxiter=1000,
         following meaning:
 
             - 0: minimum detected in the annealing process.
-            - 1: detection occured in the local search process.
+            - 1: detection occurred in the local search process.
             - 2: detection done in the dual annealing process.
 
         If the callback implementation returns True, the algorithm will stop.
     x0 : ndarray, shape(n,), optional
-        Coordinates of a single n-dimensional starting point.
+        Coordinates of a single N-D starting point.
+    local_search_options : dict, optional
+        Backwards compatible flag for `minimizer_kwargs`, only one of these
+        should be supplied.
+
+        .. deprecated:: 1.8.0
+            dual_annealing argument `local_search_options` is deprecated in
+            favor of `minimizer_kwargs` and will be removed in SciPy 1.10.0.
 
     Returns
     -------
@@ -579,11 +599,12 @@ def dual_annealing(func, bounds, args=(), maxiter=1000,
         Simulated Annealing for Efficient Global Optimization: the GenSA
         Package for R. The R Journal, Volume 5/1 (2013).
     .. [6] Mullen, K. Continuous Global Optimization in R. Journal of
-        Statistical Software, 60(6), 1 - 45, (2014). DOI:10.18637/jss.v060.i06
+        Statistical Software, 60(6), 1 - 45, (2014).
+        :doi:`10.18637/jss.v060.i06`
 
     Examples
     --------
-    The following example is a 10-dimensional problem, with many local minima.
+    The following example is a 10-D problem, with many local minima.
     The function involved is called Rastrigin
     (https://en.wikipedia.org/wiki/Rastrigin_function)
 
@@ -591,14 +612,20 @@ def dual_annealing(func, bounds, args=(), maxiter=1000,
     >>> func = lambda x: np.sum(x*x - 10*np.cos(2*np.pi*x)) + 10*np.size(x)
     >>> lw = [-5.12] * 10
     >>> up = [5.12] * 10
-    >>> ret = dual_annealing(func, bounds=list(zip(lw, up)), seed=1234)
-    >>> print("global minimum: xmin = {0}, f(xmin) = {1:.6f}".format(
-    ...       ret.x, ret.fun))
-    global minimum: xmin = [-4.26437714e-09 -3.91699361e-09 -1.86149218e-09 -3.97165720e-09
-     -6.29151648e-09 -6.53145322e-09 -3.93616815e-09 -6.55623025e-09
-    -6.05775280e-09 -5.00668935e-09], f(xmin) = 0.000000
+    >>> ret = dual_annealing(func, bounds=list(zip(lw, up)))
+    >>> ret.x
+    array([-4.26437714e-09, -3.91699361e-09, -1.86149218e-09, -3.97165720e-09,
+           -6.29151648e-09, -6.53145322e-09, -3.93616815e-09, -6.55623025e-09,
+           -6.05775280e-09, -5.00668935e-09]) # random
+    >>> ret.fun
+    0.000000
 
-    """  # noqa: E501
+    """
+
+    if isinstance(bounds, Bounds):
+        bounds = new_bounds_to_old(bounds.lb, bounds.ub, len(bounds.lb))
+
+    # noqa: E501
     if x0 is not None and not len(x0) == len(bounds):
         raise ValueError('Bounds size does not match x0')
 
@@ -621,10 +648,24 @@ def dual_annealing(func, bounds, args=(), maxiter=1000,
 
     # Wrapper for the objective function
     func_wrapper = ObjectiveFunWrapper(func, maxfun, *args)
-    # Wrapper fot the minimizer
+    # Wrapper for the minimizer
+    if local_search_options and minimizer_kwargs:
+        raise ValueError("dual_annealing only allows either 'minimizer_kwargs' (preferred) or "
+                         "'local_search_options' (deprecated); not both!")
+    if local_search_options is not None:
+        warnings.warn("dual_annealing argument 'local_search_options' is "
+                      "deprecated in favor of 'minimizer_kwargs' and will be "
+                      "removed in SciPy 1.10.0.",
+                      category=DeprecationWarning, stacklevel=2)
+        minimizer_kwargs = local_search_options
+
+    # minimizer_kwargs has to be a dict, not None
+    minimizer_kwargs = minimizer_kwargs or {}
+
     minimizer_wrapper = LocalSearchWrapper(
-        bounds, func_wrapper, **local_search_options)
-    # Initialization of RandomState for reproducible runs if seed provided
+        bounds, func_wrapper, **minimizer_kwargs)
+
+    # Initialization of random Generator for reproducible runs if seed provided
     rand_state = check_random_state(seed)
     # Initialization of the energy state
     energy_state = EnergyState(lower, upper, callback)
